﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Resources;

namespace Life
{
    class Field
    {  
        protected bool[,,] GameField;
        protected int len_x;
        protected int len_y;
        protected int count;

        // Конструктор класса
        public Field(int x, int y)
        {
            // Игровое поле и его размеры (в ячейках) 
            GameField = new bool[x, y, 2];
            len_x = x;
            len_y = y;

            // Количество живых клеток
            count = 0;
        }

        // Метод "на лету" подсчитывает количество живых клеток
        private void CountLiveCells(int x, int y, int z, bool value)
        {
            if (z != 0) return;
            bool CellStatus = GetCell(x, y, z);
            if (CellStatus == value) return;
            count = (value) ? ++count : --count;
        }

        // Метод получает значение ячейки
        protected bool GetCell(int x, int y, int z)
        {
            int true_x = x >= 0 ? x % len_x : len_x + (x % len_x);
            int true_y = y >= 0 ? y % len_y : len_y + (y % len_y);
            return GameField[true_x, true_y, z];
        }

        // Метод устанавливает значение ячейки
        protected void SetCell(int x, int y, int z, bool value)
        {
            int true_x = x >= 0 ? x % len_x : len_x + (x % len_x);
            int true_y = y >= 0 ? y % len_y : len_y + (y % len_y);
            CountLiveCells(true_x, true_y, z, value);
            GameField[true_x, true_y, z] = value;
        }

        // Метод определяет статус ячейки при смене поколений
        private bool GetNextCellGeneration(int x, int y)
        {
            int live_cells = 0;
            bool return_status = false;
            
            bool xm_ym = GetCell(x - 1, y - 1, 0);
            bool x0_ym = GetCell(x, y - 1, 0);
            bool xp_ym = GetCell(x + 1, y - 1, 0);
            bool xm_y0 = GetCell(x - 1, y, 0);
            bool x0_y0 = GetCell(x, y, 0);
            bool xp_y0 = GetCell(x + 1, y, 0);
            bool xm_yp = GetCell(x - 1, y + 1, 0);
            bool x0_yp = GetCell(x, y + 1, 0);
            bool xp_yp = GetCell(x + 1, y + 1, 0);

            live_cells = xm_ym ? ++live_cells : live_cells;
            live_cells = x0_ym ? ++live_cells : live_cells;
            live_cells = xp_ym ? ++live_cells : live_cells;
            live_cells = xm_y0 ? ++live_cells : live_cells;
            live_cells = xp_y0 ? ++live_cells : live_cells;
            live_cells = xm_yp ? ++live_cells : live_cells;
            live_cells = x0_yp ? ++live_cells : live_cells;
            live_cells = xp_yp ? ++live_cells : live_cells;

            if (x0_y0)
                return_status = (live_cells == 2 || live_cells == 3) ? true : return_status;
            else
                return_status = (live_cells == 3) ? true : return_status;

            return return_status;
        }

        // Метод меняет поколение
        protected void SetNextGeneration()
        {
            bool CellStatus, CellStatusNext;

            // Получаем состояние всех ячеек поля на следующий ход
            // и записываем в резервный слой игрового поля
            for (int count_y = 0; count_y < len_y; count_y++)
            {
                for (int count_x = 0; count_x < len_x; count_x++)
                {
                    CellStatusNext = GetNextCellGeneration(count_x, count_y);
                    SetCell(count_x,  count_y, 1, CellStatusNext);
                }
            }

            // Переносим состояние ячеек из резервного слоя в действующий
            // В резервный слой записываем признак перерисовки ячеек
            for (int count_y = 0; count_y < len_y; count_y++)
            {
                for (int count_x = 0; count_x < len_x; count_x++)
                {
                    CellStatus = GetCell(count_x, count_y, 0);
                    CellStatusNext = GetCell(count_x, count_y, 1);

                    if (CellStatus != CellStatusNext)
                    {
                        SetCell(count_x, count_y, 0, CellStatusNext);
                        SetCell(count_x, count_y, 1, true);
                    }
                    else
                        SetCell(count_x, count_y, 1, false);
                }
            }
        }

        // Метод очищает игровое поле
        protected void ClearGameField()
        {
            bool CellStatus;

            for (int count_y = 0; count_y < len_y; count_y++)
            {
                for (int count_x = 0; count_x < len_x; count_x++)
                {
                    CellStatus = GetCell(count_x, count_y, 0); 
                    
                    if (CellStatus)
                        SetCell(count_x, count_y, 0, false);

                    SetCell(count_x, count_y, 1, false);
                }
            }
        }

        // Метод получает численность живых клеток на текущий момент
        public int GetPopulation()
        {
            return count;
        }
    }
}
