﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Drawing;

namespace Life
{
    class GraphField : Field
    {
        protected Image imScrCopy;
        protected Graphics grScrCopy;
        protected Image Sprite;
        protected int SpriteSize;

        public GraphField(int x, int y, int sz) : base(x, y) 
        {

            // Спрайт (живая клетка), размер спрайта (пикселей)
            Sprite = Life.Properties.Resources.status;
            SpriteSize = sz;
            
            // Картинка-буфер игрового поля
            imScrCopy = new Bitmap(len_x * SpriteSize, len_y * SpriteSize);
            grScrCopy = Graphics.FromImage(imScrCopy);
        }

        private void PaintCell(Graphics grScreen, int x, int y, bool Screen = true)
        {
            // Получаем состояние текущей ячейки
            bool CellStatus = GetCell(x, y, 0);

            // Получаем координаты сторон ячейки (в пикселях)
            int left = x * SpriteSize;
            int top = y * SpriteSize;
            int right = (x * SpriteSize) + SpriteSize - 1;
            int bottom = (y * SpriteSize) + SpriteSize - 1;

            // Получаем холст, кисть, область рисования
            Pen MainPen = new Pen(Color.Silver);
            Rectangle rec = new Rectangle(left, top, SpriteSize - 1, SpriteSize - 1);

            // В зависимости от состояния ячейки закрашиваем клетку
            if (CellStatus)
                grScrCopy.DrawImage(Sprite, rec);
            else
                grScrCopy.FillRectangle(Brushes.White, rec);

            // Обводим клетку линиями
            if (x == 0)
                grScrCopy.DrawLine(MainPen, left, top, left, bottom);

            if (y == 0)
                grScrCopy.DrawLine(MainPen, left, top, right, top);

            grScrCopy.DrawLine(MainPen, left, bottom, right, bottom);
            grScrCopy.DrawLine(MainPen, right, top, right, bottom);

            // Переносим картинку на экран
            if (Screen)
                grScreen.DrawImage(imScrCopy, left, top, rec, GraphicsUnit.Pixel);

            // Освобождение ресурсов
            MainPen.Dispose();
        }

        private void PaintChanges(Graphics grScreen) 
        {
            int count_x, count_y;
            bool ChangeCell;

            for (count_y = 0; count_y < len_y; count_y++)
            {
                for (count_x = 0; count_x < len_x; count_x++)
                {
                    ChangeCell = GetCell(count_x, count_y, 1);

                    if (ChangeCell)
                        PaintCell(grScreen, count_x, count_y);
                }
            }    
        }

        public void MouseClick(Graphics grScreen, int x, int y)
        {
            int cell_x = x / SpriteSize;
            int cell_y = y / SpriteSize;
            bool CellStatus = GetCell(cell_x, cell_y, 0);

            if (CellStatus)
                SetCell(cell_x, cell_y, 0, false);
            else
                SetCell(cell_x, cell_y, 0, true);

            PaintCell(grScreen, cell_x, cell_y);
        }

        public void MouseClickShape(Graphics grScreen, int x, int y, int[,] coords)
        {
            MouseClick(grScreen, x, y);
            int coords_length = coords.GetLength(0);

            for (int counter = 0; counter < coords_length; counter++)
            {
                MouseClick(grScreen, x + coords[counter, 0], y + coords[counter, 1]);
            }
        }

        public void NextGeneration(Graphics grScreen)
        {
            SetNextGeneration();
            PaintChanges(grScreen);
        }

        public void PaintField(Graphics grScreen)
        {
            for (int count_y = 0; count_y < len_y; count_y++)
            {
                for (int count_x = 0; count_x < len_x; count_x++)
                {
                    PaintCell(grScreen, count_x, count_y, false);
                }
            }

            grScreen.DrawImage(imScrCopy, 0, 0);
        }

        public void ClearField(Graphics grScreen)
        {
            ClearGameField();
            PaintField(grScreen);
        }

        ~GraphField()
        {
            grScrCopy.Dispose();
            imScrCopy.Dispose();
            Sprite.Dispose();
        }
    }
}