﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Threading;

namespace Life
{

    public partial class MainForm : Form
    {
        private GraphField GameField;
        private Graphics grScreen;
        private Thread GameThread;

        public MainForm()
        {
            InitializeComponent();
        }

        private void MainForm_Load(object sender, EventArgs e)
        {
            // Размеры игрового поля и спрайта
            int x = 60;
            int y = 40;
            int sz = 16;

            // Готовим картинку на форме
            Picture.Width = (x * sz) + 2;
            Picture.Height = (y * sz) + 2;
            grScreen = Picture.CreateGraphics();
            CenteredGameFieldPicture();

            // Создаем и выводим игровое поле
            GameField = new GraphField(x, y, sz);
            GameField.PaintField(grScreen);

            // Создаем поток
            GameThread = new Thread(this.Run);
        }

        private void MainForm_SizeChanged(object sender, EventArgs e)
        {
            CenteredGameFieldPicture();
        }

        private void Picture_Paint(object sender, PaintEventArgs e)
        {
            GameField.PaintField(e.Graphics); 
        }

        private void Picture_MouseClick(object sender, MouseEventArgs e)
        {
            GameField.MouseClick(grScreen, e.X, e.Y);
            Population.Text = GameField.GetPopulation().ToString();
        }

        private void CenteredGameFieldPicture()
        {
            Picture.Left = ((this.Width - (Tools.Height / 2)) / 2) - (Picture.Width / 2);
            Picture.Top = ((this.Height - (Tools.Height / 2)) / 2) - (Picture.Height / 2);
        }

        private void btnClearField_Click(object sender, EventArgs e)
        {
            GameField.ClearField(grScreen);
            Population.Text = GameField.GetPopulation().ToString();
        }

        private void Run(object count)
        {
            for (int i = 0; i < (int)count; i++)
            {
                StepLabel.Text = (i + 1).ToString() + " ход из " + ((int)count).ToString();
                GameField.NextGeneration(grScreen);
                Population.Text = GameField.GetPopulation().ToString();
                Thread.Sleep(500);
            }
            StepLabel.Text = "";
        }

        private void btnStart_Click(object sender, EventArgs e)
        {
            GameThread = new Thread(this.Run);
            int Steps = Int32.Parse(StepsCountBox.Text);
            GameThread.Start(Steps);
        }

        private void btnStop_Click(object sender, EventArgs e)
        {
            if (GameThread.IsAlive == true)
            {
                GameThread.Abort();
                StepLabel.Text = "";
            }
        }

        private void MainForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (GameThread.IsAlive == true)
                GameThread.Abort();
        }

        ~MainForm()
        {
            grScreen.Dispose();
        }

    }
}
